#include "stdafx.h"
#include "resource.h"
#include "ExportWizard.h"
#include "GNFileDialog.h"
#include "FeedManagerLib.h"
#include "GreatNewsConfig.h"
#include "GMTimeLib.h"

#include "GNResourceManager.h"

#import "MSXML3.DLL"
/////////////////////////////////////////////////////////////////////////////////////////


//
// Implementation of class CPageSelect
///////////////////////////////////////////////////////////////////////////////////////////

LRESULT CExportWizard::CPageSelect::OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	ATLTRACE("CPageSelect::OnInitDialog()\n");
	GetPropertySheet().SendMessage( MM_CENTER_SHEET );

	//
	// Load channels and groups from file
	//
	try
	{
		m_pData->m_treeSelect.SubclassWindow(GetDlgItem(IDC_FEEDTREE));
		m_pData->m_treeSelect.SetTreeImageList();
		m_pData->m_treeSelect.LoadTree(LoadChannel);
		m_pData->m_treeSelect.ShowEverything();
	}
	CATCH_ALL_ERROR()

	DoDataExchange(false);

	CGNResourceManager* pResMngr = CGNSingleton<CGNResourceManager>::Instance();
	pResMngr->ApplyLanguageToWindow(m_hWnd, _T("ExportDialog_Select"));

	return 0;
} // OnInitDialog

LRESULT CExportWizard::CPageSelect::OnSelectAll(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	m_pData->m_treeSelect.CheckAll();
	return 0;
} // OnRdoimportwellknown

LRESULT CExportWizard::CPageSelect::OnUnselectAll(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	m_pData->m_treeSelect.CheckAll(false);
	return 0;
} // OnRdoExportSelectFile

CExportWizard::CPageSelect::CPageSelect(void) : CPropertyPageImpl<CPageSelect>( _T("Export to opml/xml") )
{
	ATLTRACE("CPageSelect::CPageSelect()\n");
}
int CExportWizard::CPageSelect::OnSetActive(void)
{
	SetWizardButtons(PSWIZB_NEXT );
	
	return 1; // return 0 to avoid setting this page as active
}
int CExportWizard::CPageSelect::OnKillActive(void)
{
	m_pData->m_treeSelect.NormalizeSelection();
	if(!DoDataExchange(true))
		return 0;

	return 1; // 0 to stay in the same page.
}
/////////////////////////////////////////////////////////////////////////////////////////


//
// Implementation of class CPageFileName
///////////////////////////////////////////////////////////////////////////////////////////

LRESULT CExportWizard::CPageFileName::OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	ATLTRACE("CPageFileName::OnInitDialog()\n");

	DoDataExchange(false);		

	CGNResourceManager* pResMngr = CGNSingleton<CGNResourceManager>::Instance();
	pResMngr->ApplyLanguageToWindow(m_hWnd, _T("ExportDialog_File"));

	return 0;
} // OnInitDialog

LRESULT CExportWizard::CPageFileName::OnBtnbrowse(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	CGNFileDialog dlg(true, _T("xml"), _T(""), OFN_HIDEREADONLY, 
		_T("OPML Files (opml, xml)\0*.opml;*.xml\0All Files (*.*)\0*.*\0"),
		NULL, false);

	if(dlg.DoModal() == IDOK)
	{
		SetDlgItemText(IDC_TXTFILENAME, dlg.m_szFileName);
	}

	return 0;
} // OnBtnbrowse

CExportWizard::CPageFileName::CPageFileName(void) : CPropertyPageImpl<CPageFileName>( _T("Export to opml/xml") )
{
	ATLTRACE("CPageFileName::CPageFileName()\n");
}

int CExportWizard::CPageFileName::OnSetActive(void)
{
	SetWizardButtons(PSWIZB_BACK | PSWIZB_NEXT );

	return 1; // return 0 to avoid setting this page as active
}
int CExportWizard::CPageFileName::OnKillActive(void)
{
	return 1; // 0 to stay in the same page.
}

int CExportWizard::CPageFileName::OnWizardNext()
{
	if(!DoDataExchange(true))
		return 0;

	if(m_pData->m_fileName.GetLength()==0)
	{
		MessageBox(ResManagerPtr->GetString(IDS_SPECIFYOUTPUT),_T("GreatNews"),MB_OK|MB_ICONINFORMATION);
		return 0;
	}


	bool bSucceeded = false;

	try
	{
		MSXML2::IXMLDOMDocumentPtr spDoc(_T("MSXML2.DOMDocument.3.0")); 
		MSXML2::IXMLDOMElementPtr opml = spDoc->createElement("opml");
		spDoc->appendChild(opml);
		opml->setAttribute("version","1.1");
		
		CString strNow = CGMTimeHelper::FormatRFC822Date(CGMTimeHelper::GetCurrentSysTime());

		MSXML2::IXMLDOMElementPtr head = spDoc->createElement("head");
		MSXML2::IXMLDOMElementPtr title = spDoc->createElement("title");
		title->text = _T("OPML generated by GreatNews");
		head->appendChild(title);
		MSXML2::IXMLDOMElementPtr dateCreated = spDoc->createElement("dateCreated");
		dateCreated->text = (LPCTSTR)strNow;
		head->appendChild(dateCreated);
		MSXML2::IXMLDOMElementPtr dateModified = spDoc->createElement("dateModified");
		dateModified->text = (LPCTSTR)strNow;
		head->appendChild(dateModified);
		opml->appendChild(head);
		MSXML2::IXMLDOMElementPtr body = spDoc->createElement("body");
		opml->appendChild(body);

		int nGroup=0; // 1 is the root group
		int nFeed =0;

		CTreeItem rootNode = m_pData->m_treeSelect.GetRootItem();
		CTreeItem childNode = rootNode.GetChild();
		while(childNode)
		{
			CTreeItem currentNode = childNode;
			childNode = childNode.GetNextSibling();

			if(!m_pData->m_treeSelect.GetCheckState(currentNode))
			{
				// not checked, skip this one
				continue;
			}

			// try to export feed group
			CFeedGroupTreeItem* pGroup = dynamic_cast<CFeedGroupTreeItem*>((CFeedTreeItem*)currentNode.GetData());
			if(pGroup)
			{
				nGroup++;
				FeedGroupPtr spGroup = pGroup->m_feedGroup;
				MSXML2::IXMLDOMElementPtr group = spDoc->createElement("outline");
				spGroup->SaveToXml(group);
				body->appendChild(group);
				
				// export feeds in this group
				CTreeItem feedNode = currentNode.GetChild();
				while(feedNode)
				{
					if(m_pData->m_treeSelect.GetCheckState(feedNode))
					{
						CNewsFeedTreeItem* pFeed = dynamic_cast<CNewsFeedTreeItem*>((CFeedTreeItem*)feedNode.GetData());
						if(pFeed)
						{
							nFeed++;
							NewsFeedPtr spNewsFeed = pFeed->m_newsFeed;
							MSXML2::IXMLDOMElementPtr feed = spDoc->createElement("outline");
							spNewsFeed->SaveToXml(feed);
							group->appendChild(feed);
						}
					}

					feedNode = feedNode.GetNextSibling();
				}
				continue;
			}

			// try to export feed
			CNewsFeedTreeItem* pFeed = dynamic_cast<CNewsFeedTreeItem*>((CFeedTreeItem*)currentNode.GetData());
			if(pFeed)
			{
				nFeed++;
				NewsFeedPtr spNewsFeed = pFeed->m_newsFeed;
				MSXML2::IXMLDOMElementPtr feed = spDoc->createElement("outline");
				spNewsFeed->SaveToXml(feed);
			
				body->appendChild(feed);
			}
		}

		//
		// Export watches and Labels?
		//
		if(m_pData->m_exportWatchLabel)
		{
			// news watches
			MSXML2::IXMLDOMElementPtr spWatchList = spDoc->createElement("watchList");
			NewsWatchVector watches;
			CNewsWatch::GetAllNewsWatches(watches, CNewsWatch::SortByDisplayOrder);
			for(NewsWatchVector::iterator it=watches.begin();it!=watches.end();it++)
			{
				NewsWatchPtr watch = *it;
				MSXML2::IXMLDOMElementPtr spWatch = spDoc->createElement("watch");
				watch->SaveToXml(spWatch);
				spWatchList->appendChild(spWatch);
			}
			opml->appendChild(spWatchList);

			// labels;
			MSXML2::IXMLDOMElementPtr spLabelList = spDoc->createElement(_T("labelList"));
			TagVector tags;
			CTag::GetAllTags(tags);
			for(TagVector::iterator it=tags.begin();it!=tags.end();it++)
			{
				TagPtr tag = *it;
				MSXML2::IXMLDOMElementPtr spTag = spDoc->createElement("label");
				tag->SaveToXml(spTag);
				spLabelList->appendChild(spTag);
			}
			opml->appendChild(spLabelList);
		}


		//
		//
		//
		spDoc->save((LPCTSTR)m_pData->m_fileName);

		CString msg;
		msg.Format(ResManagerPtr->GetString(IDS_EXPORTSUMMARY), nFeed, nGroup);
		MessageBox(msg,_T("GreatNews"),MB_OK|MB_ICONINFORMATION);
		bSucceeded = true;
	}
	CATCH_ALL_ERROR()

	if(bSucceeded)
		SetWizardButtons(PSWIZB_BACK | PSWIZB_FINISH /*PSWIZB_DISABLEDFINISH*/ );

	return -1;

}

INT_PTR CExportWizard::CPageFileName::OnWizardFinish(void)
{
	return 1;
}

/////////////////////////////////////////////////////////////////////////////////////////
