// MainDlg.cpp : implementation of the CMainDlg class
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"

#include "MainDlg.h"
#include "GNUtil.h"
#include "UIUtil.h"
#include "FeedManagerLib.h"
#include "iniFile.h"

#define WHATSNEW_URL _T("http://www.curiostudio.com/whatsnew.txt")

#define SETUP_URL _T("http://www.curiostudio.com/dist/GreatNewsSetup.exe")
#define ZIP_URL _T("http://www.curiostudio.com/dist/GreatNews.zip")


BOOL CMainDlg::PreTranslateMessage(MSG* pMsg)
{
	return CWindow::IsDialogMessage(pMsg);
}

BOOL CMainDlg::OnIdle()
{
	return FALSE;
}

LRESULT CMainDlg::OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	CIniFile iniFile;
	iniFile.SetFilename(_T("./GreatNews.ini"));
	iniFile.GetInt(_T("ActiveUpdater"), _T("Packaging"), m_nPackaging);

	// center the dialog on the screen
	CenterWindow();

	// set icons
	HICON hIcon = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
		IMAGE_ICON, ::GetSystemMetrics(SM_CXICON), ::GetSystemMetrics(SM_CYICON), LR_DEFAULTCOLOR);
	SetIcon(hIcon, TRUE);
	HICON hIconSmall = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
		IMAGE_ICON, ::GetSystemMetrics(SM_CXSMICON), ::GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
	SetIcon(hIconSmall, FALSE);

	// register object for message filtering and idle updates
	CMessageLoop* pLoop = _Module.GetMessageLoop();
	ATLASSERT(pLoop != NULL);
	pLoop->AddMessageFilter(this);
	pLoop->AddIdleHandler(this);

	UIAddChildWindowContainer(m_hWnd);

	CEdit edt = GetDlgItem(IDC_WHATSNEW);
	if(m_font.CreateFont(
			16,                        // nHeight
			0,                         // nWidth
			0,                         // nEscapement
			0,                         // nOrientation
			FW_NORMAL,                 // nWeight
			FALSE,                     // bItalic
			FALSE,                     // bUnderline
			0,                         // cStrikeOut
			ANSI_CHARSET,              // nCharSet
			OUT_DEFAULT_PRECIS,        // nOutPrecision
			CLIP_DEFAULT_PRECIS,       // nClipPrecision
			DEFAULT_QUALITY,           // nQuality
			DEFAULT_PITCH | FF_SWISS,  // nPitchAndFamily
			_T("Courier New")))     // lpszFacename
	{
		edt.SetFont(m_font);
	}


	// get what's new file
	TCHAR downloadFile[MAX_PATH];
	HRESULT hr = ::URLDownloadToCacheFile(NULL, 
											WHATSNEW_URL,
											downloadFile,
											MAX_PATH,
											0,
											NULL);

	if(hr != S_OK)
	{
		edt.SetWindowText(_T("Failed to reterieve What's New infomation"));
	}
	else
	{
		CString txt;
		CGNUtil::ReadStringFromFile(downloadFile, txt);
		txt.Replace(_T("\n"), _T("\r\n"));
		edt.SetWindowText(txt);
	}

	m_progress = GetDlgItem(IDC_PROGRESS);
	m_msg = GetDlgItem(IDC_MSG);

	m_bDownloading = false;

	DoDataExchange(FALSE);

	m_linkWhatsNew.SubclassWindow(GetDlgItem(IDC_WHATSNEWLINK));
	m_linkWhatsNew.SetHyperLink(_T("http://www.curiostudio.com/whatsnew.html"));

	m_linkHowToUpgrade.SubclassWindow(GetDlgItem(IDC_HOWTOUPGRADE));
	m_linkHowToUpgrade.SetHyperLink(_T("http://www.curiostudio.com/blog/14"));

	return TRUE;

}

LRESULT CMainDlg::OnDownload(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	DoDataExchange(TRUE);
	CIniFile iniFile;
	iniFile.SetFilename(_T("./GreatNews.ini"));
	iniFile.PutInt(_T("ActiveUpdater"), _T("Packaging"), m_nPackaging);

	if(!m_bDownloading)
	{
		// start download
		m_progress.SetRange(0,100);
		m_progress.SetPos(0);
		m_download.m_url = (m_nPackaging == 0) ? SETUP_URL : ZIP_URL;
		m_download.m_hwndToNotify = m_hWnd;
		m_download.m_bAbortDownload = false;
		m_download.Start();

		m_bDownloading = true;
		m_progress.ShowWindow(SW_SHOW);
		m_msg.ShowWindow(SW_SHOW);

		GetDlgItem(ID_DOWNLOAD).SetWindowText(_T("Stop Download"));
	}
	else
	{
		ShutdownDownload();

		GetDlgItem(ID_DOWNLOAD).SetWindowText(_T("Download && Install"));
	}

	return 0;
}

LRESULT CMainDlg::OnCancel(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	CloseDialog(wID);
	return 0;
}

void CMainDlg::CloseDialog(int nVal)
{
	ShutdownDownload();

	DestroyWindow();
	::PostQuitMessage(nVal);
}

LRESULT CMainDlg::OnDownloadReady(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	m_progress.SetPos(100);
	CUIUtil::ShowMyWindow(m_hWnd);

	m_bDownloading = false;
	MessageBox(_T("Download is ready. Automatic Update is going to close GreatNews and upgrade to the latest version.")
		, _T("GreatNews Automatic Update"), MB_OK|MB_ICONINFORMATION);

	// close 
	HWND mainGreatNews;
	if(NULL != (mainGreatNews = ::FindWindow(_T("GreatNews_FrameWindowClass"), NULL)))
	{
		::PostMessage(mainGreatNews, WM_CLOSE, 0,0);
	}

	::ShellExecute(m_hWnd, _T("open"), m_download.m_downloadedFile, NULL, NULL, SW_SHOW);

	// shut down
	CloseDialog(IDOK);
	
	return 0;
}

LRESULT CMainDlg::OnDownloadFailed(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	MessageBox(m_download.m_IEStatusMsg);
	return 0;
}

LRESULT CMainDlg::OnDownloadProgress(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
    CComCritSecLock<CComAutoCriticalSection> lock(m_download.m_lock);
	m_msg.SetWindowText(m_download.m_szCustomStatusMsg);	
	m_progress.SetPos(m_download.m_progress);

	return false;
}

void CMainDlg::ShutdownDownload()
{
	if(m_bDownloading)
	{
		m_download.m_bAbortDownload = true;
		::WaitForSingleObjectEx(m_download.GetThreadHandle(), 5000, FALSE);
		m_bDownloading = false;
	}
}
